from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.core.management import call_command
from django.core.management.base import BaseCommand
from meleager.models import (Author, Book, City, Comment, Description,
                             Language, Name, Passage, Scholium, Text, Work)
from meleager_user.models import User


class Command(BaseCommand):
    help = "Inserts dummy content in the database"

    def handle(self, *args, **options):
        call_command("flush", "--noinput")
        fixture_file = settings.BASE_DIR / "meleager" / "fixtures" / "test_data.json"
        with fixture_file.open("w") as fp:
            create_fixtures()
            call_command(
                "dumpdata",
                "meleager",
                "auth",
                "meleager_user",
                "--natural-primary",
                "--natural-foreign",
                "--indent=2",
                stdout=fp,
            )


def create_fixtures():
    # CREATE LANGUAGES
    Language.objects.all().delete()
    languages = (
        ("eng", "English"),
        ("fra", "French"),
        ("grc", "Ancient Greek (to 1453)"),
        ("ita", "Italian"),
        ("por", "Portuguese"),
        ("spa", "Spanish"),
        ("lat", "Latin"),
    )

    for l in languages:
        Language.objects.create(code=l[0], iso_name=l[1], preferred=True)

    # GROUPS AND PERMISSIONS
    editors, _ = Group.objects.get_or_create(name="Editors")
    admin_editors, _ = Group.objects.get_or_create(name="Admin editors")
    perm_own = Permission.objects.get(codename="manage_own_passages")
    perm_all = Permission.objects.get(codename="manage_all_passages")
    perm_create = Permission.objects.get(codename="can_create_mlgr_content")
    editors.permissions.add(perm_own, perm_create)
    admin_editors.permissions.add(perm_all, perm_create)

    User.objects.all().delete()
    superuser = User.objects.create_user(
        "superuser", "admin@admin.com", "superuser", is_superuser=True, is_staff=True
    )
    admin_editor = User.objects.create_user(
        "editor_admin", "editor@admin.com", "editor_admin"
    )
    admin_editor.groups.add(admin_editors)
    editor = User.objects.create_user("editor", "editor@editor.com", "editor")
    editor.groups.add(editors)

    # WORK
    work = Work.objects.create(creator=editor, last_editor=editor)
    work_descriptions = (
        Description.objects.create(description="Work desc FR", language_id="fra"),
        Description.objects.create(description="Work desc EN", language_id="eng"),
    )
    work.descriptions.add(*work_descriptions)

    # BOOK
    book = Book.objects.create(work=work, number=42)
    book.descriptions.add(
        Description.objects.create(description="Book desc GRC", language_id="grc")
    )

    # CITY
    city = City.objects.create(longitude=42, latitude=-12, unique_id=12345)
    city.names.add(Name.objects.create(name="City name EN", language_id="eng"))
    city.descriptions.add(
        Description.objects.create(description="City desc FR", language_id="eng")
    )

    # AUTHOR
    author = Author.objects.create(city_born=city, unique_id=22222)
    author.names.add(Name.objects.create(name="Author ITA", language_id="ita"))
    author.descriptions.add(
        Description.objects.create(description="Author desc EN", language_id="eng")
    )

    # PASSAGE
    passage = Passage.objects.create(book=book, fragment=12, unique_id=66666)
    passage.descriptions.add(
        Description.objects.create(description="Passage 1 desc FR", language_id="fra")
    )
    passage.authors.add(author)

    # COMMENT
    comment = Comment.objects.create(unique_id=333333)
    comment.descriptions.add(
        Description.objects.create(description="Comment passage FR", language_id="fra")
    )
    passage.comments.add(comment)

    passage2 = Passage.objects.create(book=book, fragment=69, sub_fragment="abc", unique_id=444444)
    passage2.descriptions.add(
        Description.objects.create(description="Passage 2 desc EN", language_id="eng")
    )
    passage2.authors.add(author)
    passage2.cities.add(city)

    # SCHOLIUM
    scholium = Scholium.objects.create(passage=passage2, number=666, unique_id=777777)
    scholium.descriptions.add(
        Description.objects.create(description="Scholium desc GRC", language_id="grc")
    )

    passage.scholia.add(scholium)
