import re
from urllib.parse import urlparse
from pathlib import Path

from django.apps import apps
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _

from simple_history.models import HistoricalRecords

from .abstract import (
    AbstractDescriptableResource,
    AbstractEditableResource,
    AbstractValidableResource,
    AbstractUniqueIdentifier,
    AlternativeURNs,
)


class PassageQuerySet(models.QuerySet):
    def get_by_natural_key(self, fragment, sub_fragment, *book_natural_key):
        book = apps.get_model("meleager", "Book").objects.get_by_natural_key(
            *book_natural_key
        )
        return self.get(fragment=fragment, sub_fragment=sub_fragment, book=book)

    def easy_get(self, *args):
        if not args:
            return None

        values = list()

        if type(args[0]) == str:
            values = args[0].split(".")
            if not values[-1].isdigit() and len(values) < 3:
                match = re.match(r"(\d+)(\w+)", values[-1])
                if match:
                    values[-1] = match[1]
                    values.append(match[2])
                else:
                    raise AttributeError("No match found for %s" % args[0])
        elif len(args) >= 2:
            values = list(args)
        else:
            raise AttributeError("Invalid attributes")

        if len(values) == 2:
            values.append("")

        book_number, fragment, sub_fragment = int(values[0]), int(values[1]), values[2]

        return self.get(
            book__number=book_number, fragment=fragment, sub_fragment=sub_fragment
        )


class Passage(
    AbstractEditableResource,
    AbstractValidableResource,
    AbstractDescriptableResource,
    AlternativeURNs,
    AbstractUniqueIdentifier,
    models.Model,
):
    """ Passage model """

    objects = PassageQuerySet.as_manager()
    history = HistoricalRecords()

    authors = models.ManyToManyField(
        "meleager.Author", through="meleager.AuthorPassage", related_name="passages"
    )

    cities = models.ManyToManyField(
        "meleager.City",
        related_name="passages",
    )

    book = models.ForeignKey(
        "meleager.Book",
        related_name="passages",
        help_text="Book the passage is in",
        on_delete=models.CASCADE,
        db_index=True,
    )

    fragment = models.IntegerField(
        help_text="The fragment number in book.", db_index=True
    )

    sub_fragment = models.CharField(
        help_text="The sub fragment number in book.",
        blank=True,
        null=False,
        default="",
        max_length=3,
        db_index=True,
    )

    images = models.ManyToManyField(
        "meleager.Medium",
        related_name="passages",
        help_text="The images associated to this resource.",
    )

    comments = models.ManyToManyField(
        "meleager.Comment",
        related_name="passages",
        help_text="The comments of this resource.",
    )

    texts = models.ManyToManyField(
        "meleager.Text",
        help_text="Text for this passage",
        related_name="passages",
        through="meleager.TextPassage",
    )

    external_references = models.ManyToManyField(
        "meleager.ExternalReference", related_name="passages"
    )
    internal_references = models.ManyToManyField("self", symmetrical=True)

    manuscripts = models.ManyToManyField(
        "meleager.Manuscript",
        help_text="The image of the manuscript for this passage.",
        related_name="passages",
    )

    keywords = models.ManyToManyField(
        "meleager.Keyword",
        related_name="passages",
        help_text="The keywords that tag this resource.",
        blank=True,
    )

    def check_user_perms(self, user):
        return (
            user.has_perm("meleager.manage_all_passages")
            or user == self.creator
            and user.has_perm("meleager.manage_own_passages")
        )

    def _get_url(self, **kwargs):
        if kwargs.get("next", False):
            rule_1 = models.Q(book__number__gt=self.book.number)
            rule_2 = models.Q(book__number=self.book.number) & models.Q(
                fragment__gt=self.fragment
            )
            order = ("book__number", "fragment")
        elif kwargs.get("previous", False):
            rule_1 = models.Q(book__number__lt=self.book.number)
            rule_2 = models.Q(book__number=self.book.number) & models.Q(
                fragment__lt=self.fragment
            )
            order = ("-book__number", "-fragment")
        else:
            return reverse(
                "web:passage-detail",
                args=(self.book.number, self.fragment, self.sub_fragment),
            )

        combined = rule_1 | rule_2
        passages = Passage.objects.filter(combined).order_by(*order)
        if not passages:
            return ""

        wanted = passages.values("book__number", "fragment", "sub_fragment")[0]

        return reverse(
            "web:passage-detail",
            args=(wanted["book__number"], wanted["fragment"], wanted["sub_fragment"]),
        )

    def get_previous_url(self):
        return self._get_url(previous=True)

    def get_next_url(self):
        return self._get_url(next=True)

    def get_absolute_url(self):
        return self._get_url()

    @property
    def urn_value(self):
        return Path(urlparse(self.get_absolute_url()).path).name

    def natural_key(self):
        return (self.fragment, self.sub_fragment) + self.book.natural_key()

    natural_key.dependencies = ["meleager.Work", "meleager.Book", "meleager.Author"]

    def __str__(self):
        return _("Passage %(book)s.%(fragment)s%(sub_fragment)s") % {
            "book": self.book.number,
            "fragment": self.fragment,
            "sub_fragment": f".{self.sub_fragment}" if self.sub_fragment else "",
        }

    class Meta:
        permissions = [
            ("manage_own_passages", "Can manage (edit/delete) their own Passage items"),
            ("manage_all_passages", "Can manage (edit/delete) all Passage items"),
            (
                "can_create_mlgr_content",
                "Can create Meleager artefacts (Author, Text, etc)",
            ),
        ]
